@@ -38,8 +38,6 @@ extern "C" {
 }
 
 
-#define DEFAULT_MMS_BUFFER_SIZE         65536
-
 InternetRadioDecoder::InternetRadioDecoder(QObject                    *event_parent,
                                            const InternetRadioStation &rs,
 #ifdef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
@@ -52,7 +50,9 @@ InternetRadioDecoder::InternetRadioDecoder(QObject                    *event_par
                                            int                         max_buffers,
                                            int                         max_singleBufferSize,
                                            int                         max_probe_size_bytes,
-                                           float                       max_analyze_secs)
+                                           float                       max_analyze_secs,
+                                           int                         max_retries
+                                          )
 :   m_decoderOpened       (false),
     m_av_pFormatCtx       (NULL),
     m_av_pFormatCtx_opened(false),
@@ -94,7 +94,8 @@ InternetRadioDecoder::InternetRadioDecoder(QObject                    *event_par
     m_contentType(contentType),
 #endif
     m_maxProbeSize  (max_probe_size_bytes > 2048 ? max_probe_size_bytes : 8192),
-    m_maxAnalyzeTime(max_analyze_secs     > 0.3  ? max_analyze_secs     : 0.8)
+    m_maxAnalyzeTime(max_analyze_secs     > 0.3  ? max_analyze_secs     : 0.8),
+    m_maxRetries    (max_retries)
 #ifdef DEBUG_DUMP_DECODER_STREAMS
     , m_debugCodedStream(NULL)
     , m_debugDecodedStream(NULL)
@@ -126,7 +127,7 @@ InternetRadioDecoder::~InternetRadioDecoder()
 #ifdef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
 void InternetRadioDecoder::selectStreamFromPlaylist()
 {
-    int retries = 2;
+    int retries = m_maxRetries;
     unsigned int start_play_idx = (unsigned int) rint((m_playListURLs.size()-1) * (float)rand() / (float)RAND_MAX);
     int n = m_playListURLs.size();
     for (int i = 0; !m_decoderOpened && i < n; ++i) {
@@ -149,6 +150,18 @@ void InternetRadioDecoder::selectStreamFromPlaylist()
 
 bool InternetRadioDecoder::decodePacket(AVPacket &pkt, int &processed_input_bytes)
 {
+
+#ifdef DEBUG_EXIT_ON_EXAMPLE_ERROR_PACKET
+    if (m_debugPacketCount == 29289) {
+        printf("packet %zi: (buf: %p)\n", m_debugPacketCount, pkt.data);
+        for (int i = 0; i < 8 ; ++i) {
+            printf("%02x ", (unsigned char)pkt.data[i]);
+        }
+        printf("\n");
+        exit (-1);
+    }
+#endif
+
     char *output_buf             = NULL;
     int   generated_output_bytes = 0;
           processed_input_bytes  = 0;
@@ -194,7 +207,6 @@ bool InternetRadioDecoder::decodePacket(AVPacket &pkt, int &processed_input_byte
                                                    pkt.size);
 #endif
 
-    m_encodedSize += (processed_input_bytes > 0) ? processed_input_bytes : 0;
 
 #ifdef DEBUG_DUMP_DECODER_STREAMS
     fprintf(m_debugMetaStream, "    processed input  chunk size %zi @ pos %zi\n",
@@ -207,8 +219,11 @@ bool InternetRadioDecoder::decodePacket(AVPacket &pkt, int &processed_input_byte
             (size_t)(m_decodedSize),
             (size_t)(m_decodedSize          / m_soundFormat.frameSize())
            );
+    fflush(m_debugMetaStream);
 #endif
 
+    m_encodedSize += (processed_input_bytes > 0) ? processed_input_bytes : 0;
+
 
     if (processed_input_bytes < 0) {
         /* if error, skip frame */
@@ -224,7 +239,7 @@ bool InternetRadioDecoder::decodePacket(AVPacket &pkt, int &processed_input_byte
     else if (generated_output_bytes > 0) {
 
 #ifdef DEBUG_DUMP_DECODER_STREAMS
-        fwrite  (output_buf, (size_t)generated_output_bytes, 1, m_debugDecodedStream);
+        fwrite  (output_buf, (size_t)generated_output_bytes, 1, m_debugDecodedStream); fflush(m_debugDecodedStream);
 #endif
 
         time_t cur_time = time(NULL);
@@ -244,10 +259,19 @@ bool InternetRadioDecoder::decodePacket(AVPacket &pkt, int &processed_input_byte
             updateSoundFormat();
         }
 
-//                     printf ("free buffers: %i\n", m_bufferCountSemaphore.available());
-//                     printf ("storing decoded data ...");
+
+#ifdef DEBUG_LOG_BUFFER_STATE
+        printf ("free buffers: %i\n", m_bufferCountSemaphore.available());
+        printf ("storing decoded data ...");
+#endif
+
+#ifndef DEBUG_DISCARD_DECODED_PCM_DATA
         pushBuffer(output_buf, generated_output_bytes, md, m_soundFormat);
-//                     printf (" done\n");
+#endif
+
+#ifdef DEBUG_LOG_BUFFER_STATE
+        printf (" done\n");
+#endif
 
         m_decodedSize += generated_output_bytes;
     }
@@ -310,8 +334,14 @@ void InternetRadioDecoder::run()
             if (!m_done && pkt.stream_index == m_av_audioStream) {
 
 #ifdef DEBUG_DUMP_DECODER_STREAMS
-                fprintf(m_debugMetaStream,  "received packed, size %zi\n", (size_t)pkt.size);
-                fwrite  (pkt.data, (size_t)pkt.size, 1, m_debugCodedStream);
+                m_debugPacketCount++;
+                fprintf(m_debugMetaStream,  "received packet nr. %zi (stream %i), size %zi\n", m_debugPacketCount, pkt.stream_index, (size_t)pkt.size); fflush(m_debugMetaStream);
+                fwrite  (pkt.data, (size_t)pkt.size, 1, m_debugCodedStream); fflush(m_debugCodedStream);
+                fprintf(m_debugMetaStream,  "   first 8 bytes (buf: %p): ", pkt.data);
+                for (int i = 0; i < 8 ; ++i) {
+                    fprintf(m_debugMetaStream, "%02x ", (unsigned char)pkt.data[i]);
+                }
+                fprintf(m_debugMetaStream, "\n"); fflush(m_debugMetaStream);
 #endif
 
 //                 AVDictionaryEntry *t = NULL;
@@ -336,9 +366,23 @@ void InternetRadioDecoder::run()
                 // restore original ptrs in order to allow a proper freeing
                 pkt.data = audio_pkt_org_data;
                 pkt.size = audio_pkt_org_size;
+            } else {
+#ifdef DEBUG_DUMP_DECODER_STREAMS
+                fprintf(m_debugMetaStream,  "discarded packet with size %zi\n", (size_t)pkt.size); fflush(m_debugMetaStream);
+                fprintf(m_debugMetaStream,  "   first 8 bytes (buf: %p): ", pkt.data);
+                for (int i = 0; i < 8 ; ++i) {
+                    fprintf(m_debugMetaStream, "%02x ", (unsigned char)pkt.data[i]);
+                }
+                fprintf(m_debugMetaStream, "\n"); fflush(m_debugMetaStream);
+                fflush(stdout);
+                if (m_debugPacketCount >= 29289) {
+                    exit (-1);
+                }
+#endif
             }
 
             av_free_packet(&pkt);
+            memset(&pkt, 0, sizeof(pkt));
         //         printf ("waiting for next packet\n");
         }
 
@@ -350,30 +394,52 @@ void InternetRadioDecoder::run()
 #endif
     }
 
+#ifdef DEBUG_DUMP_DECODER_STREAMS
+    fprintf(m_debugMetaStream, "left decoder thread main loop @ pos %zi, left %zi bytes in input buffer\n",
+            (size_t)m_encodedSize,
+            m_streamInputBuffer->debugBytesAvailable()
+           );
+    fflush(m_debugMetaStream);
+#endif
     thread()->exit();
 }
 
 
-void InternetRadioDecoder::pushBuffer(const char *data, size_t dataSize, const SoundMetaData &md, const SoundFormat &sf)
+void InternetRadioDecoder::pushBuffer(const char *_data, size_t dataSize, const SoundMetaData &md, const SoundFormat &orgSF)
 {
     if (m_done) {
         return;
     }
 
+    SoundFormat sf = orgSF;
+    sf.m_IsPlanar  = false;
+
+    const char *data = _data;
+    char        convBuf[dataSize];
+    if (orgSF.m_IsPlanar) {
+        data = convBuf;
+        size_t nFrames = dataSize / orgSF.frameSize();
+        orgSF.convertNonInterleavedToInterleaved(_data, convBuf, nFrames);
+    }
+
     bool    foundBuffer       = false;
-//     size_t  bufferFullSize    = 0;
-//     size_t  remainingCapacity = 0;
-//     size_t  remainingSize     = 0;
-//     size_t  nBufs             = 0;
+#ifdef DEBUG_DUMP_DECODER_STREAMS
+    size_t  bufferFullSize    = 0;
+    size_t  remainingCapacity = 0;
+    size_t  remainingSize     = 0;
+    size_t  nBufs             = 0;
+#endif
     {   QMutexLocker lock(&m_bufferAccessLock);
         if (m_buffers.size() > 0) {
             DataBuffer &buf = m_buffers.last();
             if (buf.soundFormat() == sf && buf.remainingCapacity() >= dataSize) {
                 buf.addData(data, dataSize);
-//                 bufferFullSize    = buf.fullSize();
-//                 remainingCapacity = buf.remainingCapacity();
-//                 remainingSize     = buf.remainingSize();
-//                 nBufs             = m_buffers.size();
+#ifdef DEBUG_DUMP_DECODER_STREAMS
+                bufferFullSize    = buf.fullSize();
+                remainingCapacity = buf.remainingCapacity();
+                remainingSize     = buf.remainingSize();
+                nBufs             = m_buffers.size();
+#endif
                 foundBuffer = true;
             }
         }
@@ -382,13 +448,17 @@ void InternetRadioDecoder::pushBuffer(const char *data, size_t dataSize, const S
         m_bufferCountSemaphore.acquire();
         QMutexLocker lock(&m_bufferAccessLock);
         m_buffers.push_back(DataBuffer(m_maxSingleBufferSize, data, dataSize, md, sf));
-//         DataBuffer &buf = m_buffers.last();
-//         bufferFullSize    = buf.fullSize();
-//         remainingCapacity = buf.remainingCapacity();
-//         remainingSize     = buf.remainingSize();
-//         nBufs             = m_buffers.size();
+#ifdef DEBUG_DUMP_DECODER_STREAMS
+        DataBuffer &buf = m_buffers.last();
+        bufferFullSize    = buf.fullSize();
+        remainingCapacity = buf.remainingCapacity();
+        remainingSize     = buf.remainingSize();
+        nBufs             = m_buffers.size();
+#endif
     }
-//     printf("wrote pcm buffer: nbufs=%zi, current: fullSize = %zi, remainingSize = %zi, remainingCapacity = %zi, increment = %zi\n", nBufs, bufferFullSize, remainingSize, remainingCapacity, dataSize);
+#ifdef DEBUG_DUMP_DECODER_STREAMS
+    printf("wrote pcm buffer: nbufs=%zi, current: fullSize = %zi, remainingSize = %zi, remainingCapacity = %zi, increment = %zi\n", nBufs, bufferFullSize, remainingSize, remainingCapacity, dataSize);
+#endif
 }
 
 
@@ -451,29 +521,45 @@ static int mms_read_packet(void *opaque, uint8_t *buf, int buf_size)
 static int InternetRadioDecoder_readInputBuffer(void *opaque, uint8_t *buffer, int max_size);
 #endif
 
-void InternetRadioDecoder::openAVStream(const QString &stream, bool warningsNotErrors)
+
+
+void InternetRadioDecoder::initIOCallbacks(void *opaque, int(*read_packet_func)(void *, uint8_t *, int ))
 {
-    if (m_decoderOpened) {
-        return;
-    }
+    // paranoia padding: 4x requirement
+    unsigned char *ioBuffer = reinterpret_cast<unsigned char*>(av_malloc(DEFAULT_MMS_BUFFER_SIZE + 4 * FF_INPUT_BUFFER_PADDING_SIZE));
+    
+#if  LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(52, 110, 0) // checked: avformat_open_input in ffmpeg >= 0.7
 
-    // care a bit about maximum delay when opening and autodetecting the stream:
-    // two effects:
-    //   * av_open_input_{stream,file}   has some autodetection which tries to get
-    //                                   some quite high score, which may require loading
-    //                                   e.g. 256kB (WDR5) of data... that takes time...
-    //                                   However, less data (~8kB == 1 sec @ 64kBit) should be sufficient
-    //                                   This time can be tuned by m_av_pFormatCtx->probesize
-    //   * av_find_stream_info           is also looking for some extra data before starting playback...
-    //                                   this time can be tuned by m_av_pFormatCtx->max_analyze_duration
+    m_av_byteio_contextPtr = avio_alloc_context(ioBuffer,
+                                                DEFAULT_MMS_BUFFER_SIZE,
+                                                /*write_flag = */ false,
+                                                opaque, // m_streamInputBuffer,
+                                                read_packet_func, //&InternetRadioDecoder_readInputBuffer,
+                                                NULL,
+                                                NULL
+                                               );
+    m_av_byteio_contextPtr->seekable = 0;
+
+#else
+
+    init_put_byte(&m_av_byteio_context,
+                  ioBuffer,
+                  DEFAULT_MMS_BUFFER_SIZE,            // sizeof(m_mms_buffer),
+                  false,
+                  opaque,
+                  &read_packet_func,
+                  NULL,
+                  NULL
+                 );
+
+#endif
+}
 
 
-    //av_log_set_level(255);
-    m_av_pFormatCtx = avformat_alloc_context();
-    m_av_pFormatCtx->probesize = m_maxProbeSize;
-
-    m_av_pFormatCtx->max_analyze_duration = m_maxAnalyzeTime * AV_TIME_BASE;
 
+void InternetRadioDecoder::open_av_input(AVInputFormat *iformat, const QString &stream, bool warningsNotErrors, bool use_io_context)
+{
+    
 #if  LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(52, 110, 0) // checked: avformat_open_input in ffmpeg >= 0.7
 //     AVDictionary  *av_params = NULL;
     //av_dict_set(&av_params, "key", "value", 0);
@@ -483,22 +569,79 @@ void InternetRadioDecoder::openAVStream(const QString &stream, bool warningsNotE
     av_params.prealloced_context = 1;
 #endif
 
-    m_av_pFormatCtx_opened = false;
-#ifdef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
-    m_is_mms_stream        = false;
-#endif
-    // if a format has been specified, set up the proper structures
+    int err = -1;
 
+#if  LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(52, 110, 0) // checked: avformat_open_input in ffmpeg >= 0.7
+    if (use_io_context) {
+        m_av_pFormatCtx->pb = m_av_byteio_contextPtr;
+    }
+    err = avformat_open_input(&m_av_pFormatCtx, stream.toUtf8(), iformat, NULL);
+
+    if (err != 0) { // on failure the context is automatically freed. Let's guarantee that the pointer is also nulled
+        m_av_pFormatCtx        = NULL;
+        m_av_pFormatCtx_opened = false;
+    }
+#else
+    if (use_io_context) {
+        err = av_open_input_stream(&m_av_pFormatCtx, &m_av_byteio_context, stream.toUtf8(), iformat, &av_params);
+    } else {
+        err = av_open_input_file  (&m_av_pFormatCtx, stream.toUtf8(), iformat, 0, &av_params);
+    }
+#endif
+
+    if (err != 0) {
+        if (warningsNotErrors) {
+            log(ThreadLogging::LogWarning, i18n("Could not open Stream %1").arg(stream)); //m_inputUrl.pathOrUrl()));
+        } else {
+            m_error = true;
+            log(ThreadLogging::LogError, i18n("Could not open Stream %1").arg(m_inputUrl.pathOrUrl()));
+        }
+        closeAVStream();
+    } else {
+        m_av_pFormatCtx_opened = true;
+    }
+
+#if  LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(52, 110, 0) // checked: avformat_open_input in ffmpeg >= 0.7
+//     av_dict_free(&av_params);
+#endif
+}
+
+
+
+AVInputFormat *InternetRadioDecoder::getInputFormat(const QString &fallbackFormat, bool warningsNotErrors)
+{
     LibAVGlobal::ensureInitDone();
     AVInputFormat   *iformat      = av_find_input_format(m_RadioStation.decoderClass().toLocal8Bit());
     if (!iformat) {
         QString decoderClass;
         if (m_contentType == "audio/mpeg") {
             decoderClass = "mp3";
+        } else if (m_contentType == "audio/x-mpeg") {
+            decoderClass = "mp3";
+        } else if (m_contentType == "audio/mp3") {
+            decoderClass = "mp3";
+        } else if (m_contentType == "audio/x-mp3") {
+            decoderClass = "mp3";
+        } else if (m_contentType == "video/mpeg") {
+            decoderClass = "mp3";
+        } else if (m_contentType == "video/x-mpeg") {
+            decoderClass = "mp3";
+        } else if (m_contentType == "application/x-ogg") {
+            decoderClass = "ogg";
+        } else if (m_contentType == "application/ogg") {
+            decoderClass = "ogg";
         } else if (m_contentType == "audio/ogg") {
             decoderClass = "ogg";
+        } else if (m_contentType == "video/ogg") {
+            decoderClass = "ogg";
         } else if (m_contentType == "application/flv") {
             decoderClass = "FLV";
+        } else if (m_contentType == "audio/asf") {
+            decoderClass = "asf";
+        } else if (m_contentType == "video/x-ms-asf") {
+            decoderClass = "asf";
+        } else if (m_contentType == "application/x-ms-asf-plugin") {
+            decoderClass = "asf";
         }
         if (decoderClass.length()) {
             iformat = av_find_input_format(decoderClass.toLocal8Bit());
@@ -509,33 +652,8 @@ void InternetRadioDecoder::openAVStream(const QString &stream, bool warningsNotE
     }
 
 
-
-
 #ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
 
-#if  LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(52, 110, 0) // checked: avformat_open_input in ffmpeg >= 0.7
-
-    m_av_byteio_contextPtr = avio_alloc_context((unsigned char*)av_malloc(DEFAULT_MMS_BUFFER_SIZE), //(unsigned char*)m_mms_buffer,
-                                                DEFAULT_MMS_BUFFER_SIZE,            //sizeof(m_mms_buffer),
-                                                /*write_flag = */ false,
-                                                m_streamInputBuffer,
-                                                &InternetRadioDecoder_readInputBuffer,
-                                                NULL,
-                                                NULL
-                                               );
-#else
-
-    init_put_byte(&m_av_byteio_context,
-                  (unsigned char*)av_malloc(DEFAULT_MMS_BUFFER_SIZE), // (unsigned char*)m_mms_buffer,
-                  DEFAULT_MMS_BUFFER_SIZE,            // sizeof(m_mms_buffer),
-                  false,
-                  this,
-                  &InternetRadioDecoder_readInputBuffer,
-                  NULL,
-                  NULL
-                 );
-#endif
-
     // run autodetection if format is not known
     if (!iformat) {
 
@@ -562,144 +680,20 @@ void InternetRadioDecoder::openAVStream(const QString &stream, bool warningsNotE
             log(ThreadLogging::LogDebug, i18n("Autodetected format: %1 (score %2)", iformat->long_name, score));
         }
     }
-
-    if (iformat) { // format setup / detection worked well ==> open stream
-        int err = -1;
-
-#if  LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(52, 110, 0) // checked: avformat_open_input in ffmpeg >= 0.7
-        m_av_pFormatCtx->pb = m_av_byteio_contextPtr;
-        err = avformat_open_input(&m_av_pFormatCtx, "", iformat, NULL);
-
-        if (err != 0) { // on failure the context is automatically freed. Let's guarantee that the pointer is also nulled
-            m_av_pFormatCtx        = NULL;
-            m_av_pFormatCtx_opened = false;
-        }
-#else
-        err = av_open_input_stream(&m_av_pFormatCtx, &m_av_byteio_context, "", iformat, &av_params);
-#endif
-        if (err != 0) {
-            if (warningsNotErrors) {
-                log(ThreadLogging::LogWarning, i18n("Could not open Stream %1").arg(m_inputUrl.pathOrUrl()));
-            } else {
-                m_error = true;
-                log(ThreadLogging::LogError, i18n("Could not open Stream %1").arg(m_inputUrl.pathOrUrl()));
-            }
-            closeAVStream();
-        } else {
-            m_av_pFormatCtx_opened = true;
-        }
-    }
-#else // INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
-
-
-    // load asf stream
-    if (stream.startsWith("mms://") || stream.startsWith("mmsx://")) {
-        m_is_mms_stream = true;
-        m_mms_stream = mmsx_connect(NULL, NULL, stream.toUtf8(), 1);
-        if (!m_mms_stream) {
-            if (warningsNotErrors) {
-                log(ThreadLogging::LogWarning, i18n("cannot open MMS stream %1", stream));
-            } else {
-                m_error = true;
-                log(ThreadLogging::LogError, i18n("cannot open MMS  %1", stream));
-            }
-            closeAVStream();
-        } else {
-#if  LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(52, 110, 0) // checked: avformat_open_input in ffmpeg >= 0.7
-            m_av_byteio_contextPtr = avio_alloc_context((unsigned char*)av_malloc(DEFAULT_MMS_BUFFER_SIZE), // (unsigned char*)m_mms_buffer,
-                                                        DEFAULT_MMS_BUFFER_SIZE,            // sizeof(m_mms_buffer),
-                                                        /*write_flag = */ false,
-                                                        m_mms_stream,
-                                                        &mms_read_packet,
-                                                        NULL,
-                                                        NULL
-                                                       );
-#else
-            init_put_byte(&m_av_byteio_context,
-                          (unsigned char*)av_malloc(DEFAULT_MMS_BUFFER_SIZE), // (unsigned char*)m_mms_buffer,
-                          DEFAULT_MMS_BUFFER_SIZE,            // sizeof(m_mms_buffer),
-                          false,
-                          m_mms_stream,
-                          &mms_read_packet,
-                          NULL,
-                          NULL
-                         );
-#endif
-        }
-
-        if (!iformat) {
-            iformat = av_find_input_format("asf");
-        }
-
-        if (iformat && m_mms_stream) {
-            int err = -1;
-#if  LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(52, 110, 0) // checked: avformat_open_input in ffmpeg >= 0.7
-            m_av_pFormatCtx->pb = m_av_byteio_contextPtr;
-            err = avformat_open_input(&m_av_pFormatCtx, stream.toUtf8(), iformat, NULL);
-
-            if (err != 0) { // on failure the context is automatically freed. Let's guarantee that the pointer is also nulled
-                m_av_pFormatCtx        = NULL;
-                m_av_pFormatCtx_opened = false;
-            }
-#else
-            err = av_open_input_stream(&m_av_pFormatCtx, &m_av_byteio_context, stream.toUtf8(), iformat, &av_params);
-#endif
-            if (err != 0) {
-                if (warningsNotErrors) {
-                    log(ThreadLogging::LogWarning, i18n("Could not open Stream %1").arg(stream));
-                } else {
-                    m_error = true;
-                    log(ThreadLogging::LogError, i18n("Could not open Stream %1").arg(stream));
-                }
-                closeAVStream();
-            } else {
-                m_av_pFormatCtx_opened = true;
-            }
-        }
-    }
-    else {
-//         IErrorLogClient::staticLogDebug("InternetRadioDecoder::openAVStream: av_open_input_file start");
-        int open_result = -1;
-
-#if  LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(52, 110, 0) // checked: avformat_open_input in ffmpeg >= 0.7
-        open_result = avformat_open_input(&m_av_pFormatCtx, stream.toUtf8(), iformat, NULL);
-
-        if (open_result != 0) { // on failure the context is automatically freed. Let's guarantee that the pointer is also nulled
-            m_av_pFormatCtx        = NULL;
-            m_av_pFormatCtx_opened = false;
-        }
-#else
-        open_result = av_open_input_file (&m_av_pFormatCtx, stream.toUtf8(), iformat, 0, &av_params);
+#else // INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD defined below
 #endif
 
-        if (open_result != 0) {
-            if (warningsNotErrors) {
-                log(ThreadLogging::LogWarning, i18n("Could not open Stream %1").arg(stream));
-            } else {
-                m_error = true;
-                log(ThreadLogging::LogError, i18n("Could not open Stream %1").arg(stream));
-            }
-            closeAVStream();
-        } else {
-            m_av_pFormatCtx_opened = true;
-        }
-//         IErrorLogClient::staticLogDebug("InternetRadioDecoder::openAVStream: av_open_input_file done");
+    if (!iformat && fallbackFormat.length()) {
+        iformat = av_find_input_format(fallbackFormat.toLocal8Bit());
     }
 
-//     IErrorLogClient::staticLogDebug("InternetRadioDecoder::openAVStream: av_find_stream_info start");
-#endif // INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
+    return iformat;
+}
 
 
 
-#if  LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(52, 110, 0) // checked: avformat_open_input in ffmpeg >= 0.7
-//     av_dict_free(&av_params);
-#endif
-
-    if (!m_av_pFormatCtx_opened) {
-        return;
-    }
-
-
+bool InternetRadioDecoder::retrieveStreamInformation(const QString &stream, bool warningsNotErrors)
+{
     // Retrieve stream information
     int err = -1;
 #if  LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(52, 111, 0) // checked: avformat_find_stream_info in ffmpeg >= 0.7.8
@@ -715,7 +709,7 @@ void InternetRadioDecoder::openAVStream(const QString &stream, bool warningsNotE
             log(ThreadLogging::LogError, i18n("Could not find stream information in %1").arg(stream));
         }
         closeAVStream();
-        return; // Couldn't find stream information
+        return false; // Couldn't find stream information
     }
 
 //     IErrorLogClient::staticLogDebug("InternetRadioDecoder::openAVStream: av_find_stream_info done");
@@ -783,11 +777,16 @@ void InternetRadioDecoder::openAVStream(const QString &stream, bool warningsNotE
             log(ThreadLogging::LogError, i18n("Could not find an audio stream in %1").arg(stream));
         }
         closeAVStream();
-        return;
+        return false;
     }
     updateSoundFormat();
+    return true;
+}
 
 
+
+bool InternetRadioDecoder::openCodec(const QString &stream, bool warningsNotErrors)
+{
     m_av_aCodecCtx = m_av_pFormatCtx->streams[m_av_audioStream]->codec;
 
     if (!m_av_aCodec) {
@@ -801,13 +800,15 @@ void InternetRadioDecoder::openAVStream(const QString &stream, bool warningsNotE
             log(ThreadLogging::LogError,   i18n("Could not find a codec for %1").arg(stream));
         }
         closeAVStream();
-        return;
+        return false;
     }
 
 #if  LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(52, 123, 0) // checked: avcodec_open2 in ffmpeg >= 0.7.8
-    err = avcodec_open2(m_av_aCodecCtx, m_av_aCodec, NULL);
+    AVDictionary *codecOpts = NULL;
+    av_dict_set(&codecOpts, "threads", "auto", 0);
+    int err = avcodec_open2(m_av_aCodecCtx, m_av_aCodec, &codecOpts);
 #else
-    err = avcodec_open(m_av_aCodecCtx, m_av_aCodec);
+    int err = avcodec_open(m_av_aCodecCtx, m_av_aCodec);
 #endif
     if (err < 0) {
         if (warningsNotErrors) {
@@ -817,12 +818,112 @@ void InternetRadioDecoder::openAVStream(const QString &stream, bool warningsNotE
             log(ThreadLogging::LogError, i18n("Opening codec for %1 failed").arg(stream));
         }
         closeAVStream();
-        return;
+        return false;
     }
 
     if (!m_soundFormat.isValid()) {
         updateSoundFormat();
     }
+    return true;
+}
+
+
+void InternetRadioDecoder::openAVStream(const QString &stream, bool warningsNotErrors)
+{
+    if (m_decoderOpened) {
+        return;
+    }
+
+#ifdef DEBUG_DUMP_DECODER_STREAMS
+    m_debugPacketCount = -1;
+#endif
+
+    // care a bit about maximum delay when opening and autodetecting the stream:
+    // two effects:
+    //   * av_open_input_{stream,file}   has some autodetection which tries to get
+    //                                   some quite high score, which may require loading
+    //                                   e.g. 256kB (WDR5) of data... that takes time...
+    //                                   However, less data (~8kB == 1 sec @ 64kBit) should be sufficient
+    //                                   This time can be tuned by m_av_pFormatCtx->probesize
+    //   * av_find_stream_info           is also looking for some extra data before starting playback...
+    //                                   this time can be tuned by m_av_pFormatCtx->max_analyze_duration
+
+
+    //av_log_set_level(255);
+    m_av_pFormatCtx = avformat_alloc_context();
+    m_av_pFormatCtx->probesize = m_maxProbeSize;
+
+    m_av_pFormatCtx->max_analyze_duration = m_maxAnalyzeTime * AV_TIME_BASE;
+
+    m_av_pFormatCtx_opened = false;
+#ifdef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
+    m_is_mms_stream        = false;
+#endif
+    // if a format has been specified, set up the proper structures
+
+
+
+#ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
+
+    initIOCallbacks(m_streamInputBuffer, &InternetRadioDecoder_readInputBuffer);
+
+    AVInputFormat   *iformat = getInputFormat("", warningsNotErrors);
+
+    if (iformat) { // format setup / detection worked well ==> open stream
+        open_av_input(iformat, m_inputUrl.pathOrUrl(), warningsNotErrors, /* use_io_context = */ true);
+    }
+#else // INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD defined below
+
+
+    // load asf stream
+    KUrl  streamUrl = stream;
+    if (streamUrl.protocol().startsWith("mms")) {
+        m_is_mms_stream = true;
+        m_mms_stream = mmsx_connect(NULL, NULL, stream.toUtf8(), 1);
+        if (!m_mms_stream) {
+            if (warningsNotErrors) {
+                log(ThreadLogging::LogWarning, i18n("cannot open MMS stream %1", stream));
+            } else {
+                m_error = true;
+                log(ThreadLogging::LogError, i18n("cannot open MMS  %1", stream));
+            }
+            closeAVStream();
+        } else {
+            initIOCallbacks(m_mms_stream, &mms_read_packet);
+        }
+
+        AVInputFormat   *iformat = getInputFormat("asf", warningsNotErrors);
+
+        if (iformat && m_mms_stream) {
+            open_av_input(iformat, stream, warningsNotErrors, /* use_io_context = */ true);
+        }
+    }
+    // else: here stream is not mms/mmsx
+    else {
+        AVInputFormat   *iformat = getInputFormat("", warningsNotErrors);
+
+//         IErrorLogClient::staticLogDebug("InternetRadioDecoder::openAVStream: av_open_input_file start");
+        open_av_input(iformat, stream, warningsNotErrors, /* use_io_context = */ false);
+//         IErrorLogClient::staticLogDebug("InternetRadioDecoder::openAVStream: av_open_input_file done");
+    }
+
+//     IErrorLogClient::staticLogDebug("InternetRadioDecoder::openAVStream: av_find_stream_info start");
+#endif // INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
+
+
+
+
+    if (!m_av_pFormatCtx_opened) {
+        return;
+    }
+
+    if (!retrieveStreamInformation(stream, warningsNotErrors)) {
+        return;
+    }
+
+    if (!openCodec(stream, warningsNotErrors)) {
+        return;
+    }
     m_decoderOpened = true;
 }
 
@@ -836,31 +937,52 @@ void InternetRadioDecoder::updateSoundFormat()
         m_av_pFormatCtx->streams[m_av_audioStream]      &&
         m_av_pFormatCtx->streams[m_av_audioStream]->codec
     ) {
-        int rate = m_av_pFormatCtx->streams[m_av_audioStream]->codec->sample_rate;
-        int ch   = m_av_pFormatCtx->streams[m_av_audioStream]->codec->channels;
-        int fmt  = m_av_pFormatCtx->streams[m_av_audioStream]->codec->sample_fmt;
-        int bits     = 0;
-        int issigned = 0;
+        int  rate = m_av_pFormatCtx->streams[m_av_audioStream]->codec->sample_rate;
+        int  ch   = m_av_pFormatCtx->streams[m_av_audioStream]->codec->channels;
+        int  fmt  = m_av_pFormatCtx->streams[m_av_audioStream]->codec->sample_fmt;
+        int  bits     = 0;
+        int  issigned = 0;
+        bool isplanar = false;
         switch(fmt) {
             case AV_SAMPLE_FMT_U8:
                 bits     = 8;
                 issigned = false;
+                isplanar = false;
                 break;
             case AV_SAMPLE_FMT_S16:
                 bits     = 16;
                 issigned = true;
+                isplanar = false;
                 break;
             case AV_SAMPLE_FMT_S32:
                 bits     = 32;
                 issigned = true;
+                isplanar = false;
                 break;
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(50, 15, 1) // checked: first in ffmpeg >= 0.9
+            case AV_SAMPLE_FMT_U8P:
+                bits     = 8;
+                issigned = false;
+                isplanar = true;
+                break;
+            case AV_SAMPLE_FMT_S16P:
+                bits     = 16;
+                issigned = true;
+                isplanar = true;
+                break;
+            case AV_SAMPLE_FMT_S32P:
+                bits     = 32;
+                issigned = true;
+                isplanar = true;
+                break;
+#endif
             default:
                 m_error = true;
                 log(ThreadLogging::LogError, i18n("Cannot use libav sample format id %1").arg(fmt));
                 closeAVStream();
                 return;
         }
-        m_soundFormat = SoundFormat(rate, ch, bits, issigned);
+        m_soundFormat = SoundFormat(rate, ch, bits, issigned, isplanar);
     }
 }
 
@@ -900,7 +1022,8 @@ void InternetRadioDecoder::closeAVStream()
     // otherwise the avformat_close_input function calls will do the job
     if (!m_av_pFormatCtx_opened && m_av_pFormatCtx) {
         av_free(m_av_pFormatCtx);
-        m_av_pFormatCtx = NULL;
+        m_av_pFormatCtx        = NULL;
+        m_av_pFormatCtx_opened = false;
     }
 
 #ifdef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
@@ -927,11 +1050,12 @@ void InternetRadioDecoder::closeAVStream()
     m_mms_stream     = NULL;
 #endif
 
-    m_av_pFormatCtx  = NULL;
-    m_av_audioStream = -1;
-    m_av_aCodecCtx   = NULL;
-    m_av_aCodec      = NULL;
-    m_decoderOpened  = false;
+    m_av_pFormatCtx        = NULL;
+    m_av_pFormatCtx_opened = NULL;
+    m_av_audioStream       = -1;
+    m_av_aCodecCtx         = NULL;
+    m_av_aCodec            = NULL;
+    m_decoderOpened        = false;
 }
 
 
@@ -939,12 +1063,41 @@ void InternetRadioDecoder::closeAVStream()
 
 static int InternetRadioDecoder_readInputBuffer(void *opaque, uint8_t *buffer, int max_size)
 {
+#ifdef DEBUG_DUMP_DECODER_STREAMS
+    static FILE   *debugFH = NULL;
+    static size_t  pos     = 0;
+    if (!debugFH) {
+        debugFH = fopen("/tmp/kradio-decoder-thread-input-stream.bin", "w");
+        printf("InternetRadioDecoder_readInputBuffer: opened input-stream debug file\n");
+    }
+//     if (pos >= 0x808000) {
+//         printf("dummy\n");
+//     }
+#endif
+
     StreamInputBuffer *x = static_cast<StreamInputBuffer*>(opaque);
     bool               err = false;
     QByteArray tmp = x->readInputBuffer(1024, max_size, /* consume */ true, err); // at least a kB
-//     printf ("read returned %i bytes, err = %i\n", tmp.size(), err);
     if (!err) {
-        memcpy(buffer, tmp.constData(), tmp.size());
+        const unsigned char *rxBuf   = (const unsigned char*)tmp.constData();
+        size_t               bufsize = tmp.size();
+        memcpy(buffer, rxBuf, bufsize);
+
+#ifdef DEBUG_DUMP_DECODER_STREAMS
+        printf ("input stream read @ %zi (0x%08zx), rxBuf = %p, returned %zi bytes (0x%08zx), err = %i, buf=%p, maxSize = 0x%08x\n", pos, pos, rxBuf, bufsize, bufsize, err, buffer, max_size);
+        printf ("  first 16 bytes: ");
+        for (int i = 0; i < 16; ++i) {
+            printf("%02x ", buffer[i]);
+        }
+        printf("\n"); fflush(stdout);
+        printf ("  last 17 bytes:  ");
+        for (int i = -17; i < 0; ++i) {
+            printf("%02x ", buffer[bufsize + i]);
+        }
+        printf("\n"); fflush(stdout);
+        fwrite(tmp.constData(), bufsize, 1, debugFH); fflush(debugFH);
+        pos += tmp.size();
+#endif
         return tmp.size();
     } else {
         return -1;
@@ -970,16 +1123,20 @@ DecoderThread::DecoderThread(QObject *parent,
                              StreamReader               *streamReader,
 //                              StreamInputBuffer          *streamInputBuffer,
 #endif
-                             int   max_buffers,
-                             int   max_singleBufferSize,
+                             int   input_buffer_size,
+                             int   max_output_buffers,
+                             int   max_output_buffer_chunk_size,
                              int   max_probe_size_bytes,
-                             float max_analyze_secs)
+                             float max_analyze_secs,
+                             int   max_retries
+                            )
     : QThread(parent),
       m_station(rs),
-      m_max_buffers(max_buffers),
-      m_max_singleBufferSize(max_singleBufferSize),
+      m_max_buffers(max_output_buffers),
+      m_max_singleBufferSize(max_output_buffer_chunk_size),
       m_max_probe_size_bytes(max_probe_size_bytes),
       m_max_analyze_secs(max_analyze_secs),
+      m_max_retries(max_retries),
       m_decoder(NULL)
 #ifdef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
       , m_playlist(playlist)
@@ -989,7 +1146,7 @@ DecoderThread::DecoderThread(QObject *parent,
 #endif
 {
 #ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
-    m_streamInputBuffer = new StreamInputBuffer(128 * 1024);         // FIXME: make buffer configurable
+    m_streamInputBuffer = new StreamInputBuffer(input_buffer_size);
     QObject::connect(streamReader,        SIGNAL(sigStreamData(QByteArray)), m_streamInputBuffer, SLOT(slotWriteInputBuffer(QByteArray)));
     QObject::connect(m_streamInputBuffer, SIGNAL(sigInputBufferFull()),      streamReader,        SLOT(slotStreamPause()));
     QObject::connect(m_streamInputBuffer, SIGNAL(sigInputBufferNotFull()),   streamReader,        SLOT(slotStreamContinue()));
@@ -1030,7 +1187,8 @@ void DecoderThread::run()
                                          m_max_buffers,
                                          m_max_singleBufferSize,
                                          m_max_probe_size_bytes,
-                                         m_max_analyze_secs
+                                         m_max_analyze_secs,
+                                         m_max_retries
                                         );
     exec();
     m_streamInputBuffer->resetBuffer();
