@@ -38,13 +38,8 @@
 #include "mmsx_handler.h"
 
 
-#ifdef KRADIO_ENABLE_FIXMES
-    #warning "FIXME: make buffer size configurable"
-#endif
-#define MAX_BUFFER_SIZE       (256*1024)
-#define MAX_BUFFERS           8
-#define MAX_BYTES_PER_BUFFER  ((MAX_BUFFER_SIZE) / (MAX_BUFFERS))
-#define MIN_BUFFERS4PLAYBACK  2
+#define MAX_BUFFER_CHUNKS           16
+#define MIN_BUFFER_CHUNKS4PLAYBACK  (MAX_BUFFER_CHUNKS/3)
 
 ///////////////////////////////////////////////////////////////////////
 
@@ -74,6 +69,10 @@ InternetRadio::InternetRadio(const QString &instanceID, const QString &name)
     m_RDS_RadioText(QString()),
     m_maxStreamProbeSize(8192),
     m_maxStreamAnalyzeTime(0.8),
+    m_inputBufferSize(128*1024),
+    m_outputBufferSize(512*1024),
+    m_watchdogTimeout(0),
+    m_watchdogHandlerInService(false),
     m_waitForBufferMinFill(true)
 {
     m_SoundStreamSinkID   = createNewSoundStream(false);
@@ -83,6 +82,7 @@ InternetRadio::InternetRadio(const QString &instanceID, const QString &name)
     QObject::connect(&m_playlistHandler, SIGNAL(sigError(QString)),             this, SLOT(slotPlaylistError(QString)));
     QObject::connect(&m_playlistHandler, SIGNAL(sigPlaylistLoaded(KUrl::List)), this, SLOT(slotPlaylistLoaded(KUrl::List)));
     QObject::connect(&m_playlistHandler, SIGNAL(sigStreamSelected(KUrl)),       this, SLOT(slotPlaylistStreamSelected(KUrl)));
+    QObject::connect(&m_watchdogTimer,   SIGNAL(timeout()),                     this, SLOT(slotWatchdogTimeout()));
 }
 
 
@@ -542,6 +542,27 @@ void InternetRadio::slotNoticePlaybackMixerChanged(const QString &mixerID, const
 }
 
 
+void InternetRadio::slotBufferSettingsChanged (int inputBufSize, int outputBufSize)
+{
+    m_inputBufferSize  = inputBufSize;
+    m_outputBufferSize = outputBufSize;
+}
+
+
+void InternetRadio::slotWatchdogSettingsChanged (int timeout)
+{
+    m_watchdogTimeout  = timeout;
+    m_watchdogTimer.stop();
+}
+
+
+void InternetRadio::slotDecoderSettingsChanged (int probe_size, double analysis_time)
+{
+    m_maxStreamProbeSize   = probe_size;
+    m_maxStreamAnalyzeTime = analysis_time;
+}
+
+
 // PluginBase methods
 
 void   InternetRadio::saveState (KConfigGroup &config) const
@@ -551,14 +572,15 @@ void   InternetRadio::saveState (KConfigGroup &config) const
     config.writeEntry("PlaybackMixerID",             m_PlaybackMixerID);
     config.writeEntry("PlaybackMixerChannel",        m_PlaybackMixerChannel);
     config.writeEntry("PlaybackMixerMuteOnPowerOff", m_PlaybackMixerMuteOnPowerOff);
+    config.writeEntry("InputBufferSize",             m_inputBufferSize);
+    config.writeEntry("OutputBufferSize",            m_outputBufferSize);
+    config.writeEntry("WatchdogTimeout",             m_watchdogTimeout);
     config.writeEntry("defaultPlaybackVolume",       m_defaultPlaybackVolume);
     config.writeEntry("URL",                         m_currentStation.url());
     config.writeEntry("PowerOn",                     isPowerOn());
     config.writeEntry("maxStreamProbeSizeNew",       m_maxStreamProbeSize);
     config.writeEntry("maxStreamAnalyzeTimeNew",     m_maxStreamAnalyzeTime);
-#ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
     config.writeEntry("maxStreamRetries",            m_maxStreamRetries);
-#endif
 
     saveRadioDeviceID(config);
 }
@@ -572,17 +594,21 @@ void   InternetRadio::restoreState (const KConfigGroup &config)
     QString PlaybackMixerID       = config.readEntry ("PlaybackMixerID",             QString());
     QString PlaybackMixerChannel  = config.readEntry ("PlaybackMixerChannel",        "PCM");
     bool    muteOnPowerOff        = config.readEntry ("PlaybackMixerMuteOnPowerOff", false);
+    m_inputBufferSize             = config.readEntry ("InputBufferSize",             128*1024);
+    m_outputBufferSize            = config.readEntry ("OutputBufferSize",            512*1024);
+    m_watchdogTimeout             = config.readEntry ("WatchdogTimeout",             0);
     m_defaultPlaybackVolume       = config.readEntry ("defaultPlaybackVolume",       0.5);
 
 
     m_maxStreamProbeSize          = config.readEntry ("maxStreamProbeSizeNew",       8192);
     m_maxStreamAnalyzeTime        = config.readEntry ("maxStreamAnalyzeTimeNew",     0.8);
-#ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
     m_maxStreamRetries            = config.readEntry ("maxStreamRetries",            2);
-#endif
 
 
     setPlaybackMixer(PlaybackMixerID, PlaybackMixerChannel, muteOnPowerOff, /* force = */ true);
+    emit sigBufferSettingsChanged  (m_inputBufferSize, m_outputBufferSize);
+    emit sigWatchdogSettingsChanged(m_watchdogTimeout);
+    emit sigDecoderSettingsChanged (m_maxStreamProbeSize, m_maxStreamAnalyzeTime);
 
     setURL(config.readEntry("URL", KUrl()), NULL);
     m_restorePowerOn = config.readEntry ("PowerOn", false);
@@ -591,6 +617,7 @@ void   InternetRadio::restoreState (const KConfigGroup &config)
         notifyPlaybackVolumeChanged(m_SoundStreamSinkID, m_defaultPlaybackVolume);
 }
 
+
 void InternetRadio::startPlugin()
 {
     PluginBase::startPlugin();
@@ -603,9 +630,15 @@ ConfigPageInfo InternetRadio::createConfigurationPage()
 
     QObject::connect(this, SIGNAL(sigNotifyPlaybackMixerChanged (const QString &, const QString &, bool, bool)),
                      conf, SLOT  (slotNoticePlaybackMixerChanged(const QString &, const QString &, bool, bool)));
+    QObject::connect(this, SIGNAL(sigBufferSettingsChanged  (int,int)),    conf, SLOT(slotBufferSettingsChanged  (int,int)));
+    QObject::connect(this, SIGNAL(sigWatchdogSettingsChanged(int)),        conf, SLOT(slotWatchdogSettingsChanged(int)));
+    QObject::connect(this, SIGNAL(sigDecoderSettingsChanged (int,double)), conf, SLOT(slotDecoderSettingsChanged (int,double)));
 
     QObject::connect(conf, SIGNAL(sigPlaybackMixerChanged       (const QString &, const QString &, bool, bool)),
                      this, SLOT  (slotNoticePlaybackMixerChanged(const QString &, const QString &, bool, bool)));
+    QObject::connect(conf, SIGNAL(sigBufferSettingsChanged  (int,int)),    this, SLOT(slotBufferSettingsChanged  (int,int)));
+    QObject::connect(conf, SIGNAL(sigWatchdogSettingsChanged(int)),        this, SLOT(slotWatchdogSettingsChanged(int)));
+    QObject::connect(conf, SIGNAL(sigDecoderSettingsChanged (int,double)), this, SLOT(slotDecoderSettingsChanged (int,double)));
 
     return ConfigPageInfo (conf,
                            i18n("Internet Radio"),
@@ -691,7 +724,7 @@ void InternetRadio::slotPlaylistEOL()
 void InternetRadio::startStreamReader(KUrl stream)
 {
     stopStreamReader();
-    if (stream.protocol() == "mms" || stream.protocol() == "mmsx") {
+    if (stream.protocol().startsWith("mms")) {
         m_streamReader = new MMSXHandler();
     } else {
         m_streamReader = new IcyHttpHandler();
@@ -703,7 +736,9 @@ void InternetRadio::startStreamReader(KUrl stream)
     connect(m_streamReader, SIGNAL(sigUrlChanged(KUrl)),                          this, SLOT(slotInputStreamUrlChanged(KUrl)));
     connect(m_streamReader, SIGNAL(sigConnectionEstablished(KUrl,KIO::MetaData)), this, SLOT(slotStreamConnectionEstablished(KUrl,KIO::MetaData)));
 
-    m_streamReader->startStreamDownload(stream);
+    connect(m_streamReader, SIGNAL(sigStreamData(QByteArray)),                    this, SLOT(slotWatchdogData(QByteArray)));
+
+    m_streamReader->startStreamDownload(stream, m_currentStation.metaDataEncoding());
 }
 
 
@@ -734,10 +769,12 @@ void InternetRadio::startDecoderThread()
                                         m_streamReader,
 //                                         m_streamInputBuffer,
 #endif
-                                        MAX_BUFFERS,
-                                        MAX_BYTES_PER_BUFFER,
+                                        m_inputBufferSize,
+                                        MAX_BUFFER_CHUNKS,
+                                        m_outputBufferSize / MAX_BUFFER_CHUNKS,
                                         m_maxStreamProbeSize,
-                                        m_maxStreamAnalyzeTime
+                                        m_maxStreamAnalyzeTime,
+                                        m_maxStreamRetries
                                        );
     QObject::connect(m_decoderThread, SIGNAL(finished()),   this, SLOT(slotDecoderThreadFinished()));
     QObject::connect(m_decoderThread, SIGNAL(terminated()), this, SLOT(slotDecoderThreadFinished()));
@@ -784,6 +821,7 @@ void InternetRadio::radio_done()
     m_powerOn    = false;
     m_stereoFlag = false;
 
+    m_watchdogTimer.stop();
     m_playlistHandler.stopPlaylistDownload();
 #ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
     stopStreamReader();
@@ -891,15 +929,16 @@ bool InternetRadio::noticeReadyForPlaybackData(SoundStreamID id, size_t free_siz
         return false;
     }
 
-    int min_size = m_waitForBufferMinFill ? MIN_BUFFERS4PLAYBACK : 1;
+    int min_size = m_waitForBufferMinFill ? MIN_BUFFER_CHUNKS4PLAYBACK : 1;
 
     if (!m_decoderThread || !m_decoderThread->decoder())
         return false;
 
     int n_bufs = m_decoderThread->decoder()->availableBuffers();
 
+    // printf("inet radio PLAY: n_bufs = %i, min_size = %i\n", n_bufs, min_size);
     if (n_bufs < min_size) {
-        m_waitForBufferMinFill = true;
+        // m_waitForBufferMinFill = true;
 //         logDebug(QString("InternetRadio::noticeReadyForPlaybackData: SKIP: min_count = %1, buf_count = %2").arg(min_size).arg(n_bufs));
         return false;
     }
@@ -929,8 +968,8 @@ bool InternetRadio::noticeReadyForPlaybackData(SoundStreamID id, size_t free_siz
         if (consumed_size == SIZE_T_DONT_CARE) {
             consumed_size = size;
         }
-//         printf("PLAY: buf_size = %zi, min_count = %i, buf_count = %i, free_size=%zi, avail_size=%zi\n", size, min_size, n_bufs, free_size, dataSize);
-//         printf("     consumed: %zi\n", consumed_size);
+        // printf("inet radio PLAY: buf_size = %zi, min_count = %i, buf_count = %i, free_size=%zi, avail_size=%zi\n", size, min_size, n_bufs, free_size, dataSize);
+        // printf("     consumed: %zi\n", consumed_size);
         free_size -= consumed_size;
         buf.addProcessedSize(consumed_size);
         if (m_decoderThread && buf.remainingSize() <= 0) {
@@ -1070,6 +1109,36 @@ void InternetRadio::slotMetaDataUpdate(KIO::MetaData metadata)
 }
 
 
+// #define  DEBUG_TEST_WATCHDOG
+
+void InternetRadio::slotWatchdogData(QByteArray data)
+{
+#ifdef DEBUG_TEST_WATCHDOG
+    bool reset = data.size() > 0 && !m_watchdogTimer.isActive();
+#else
+    bool reset = data.size() > 0;
+#endif
+    if (isPowerOn() && reset) {
+        m_watchdogTimer.stop();
+        if (m_watchdogTimeout > 0) {
+            m_watchdogTimer.setSingleShot(true);
+            m_watchdogTimer.start(m_watchdogTimeout * 1000);
+        }
+    }
+}
+
+
+void InternetRadio::slotWatchdogTimeout()
+{
+    if (isPowerOn() && !m_watchdogHandlerInService) {
+        m_watchdogHandlerInService = true;
+        logWarning(i18n("Internet Radio Plugin (%1): stream data timeout (>= %2 s)").arg(m_playlistHandler.currentStreamUrl().pathOrUrl()).arg(m_watchdogTimeout));
+        m_playlistHandler.selectNextStream(false, false, false);
+        m_watchdogHandlerInService = false;
+    }
+}
+
+
 
 #ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
 
@@ -1082,7 +1151,11 @@ void InternetRadio::slotStreamError(KUrl /*url*/)
 
 void InternetRadio::slotStreamFinished(KUrl /*url*/)
 {
-    powerOff();
+    if (m_watchdogTimeout) {
+        slotWatchdogTimeout();
+    } else {
+        powerOff();
+    }
 }
 
 
