@@ -50,7 +50,10 @@ TimeControlConfiguration::TimeControlConfiguration (QWidget *parent)
       ITimeControlClient(),
       IRadioClient(),
       ignoreChanges(false),
-      m_dirty(false)
+      m_dirty(false),
+      m_enabledAlarmTextForeground(Qt::black),
+      m_disabledAlarmTextForeground(Qt::gray),
+      m_defaultAlarmTextForegroundValid(false)
 {
     setupUi(this);
 
@@ -85,7 +88,10 @@ TimeControlConfiguration::TimeControlConfiguration (QWidget *parent)
     QObject::connect(buttonAlarmNew,        SIGNAL(clicked()),                   this, SLOT(slotNewAlarm()));
     QObject::connect(buttonDeleteAlarm,     SIGNAL(clicked()),                   this, SLOT(slotDeleteAlarm()));
     QObject::connect(comboAlarmType,        SIGNAL(highlighted(int)),            this, SLOT(slotAlarmTypeChanged(int)));
-    QObject::connect(editRecordingTemplate, SIGNAL(textEdited(const QString &)), this, SLOT(slotRecordingTemplateChanged(const QString &)));
+    QObject::connect(editRecordingTemplateFileName,  SIGNAL(textEdited(const QString &)), this, SLOT(slotRecordingTemplateFilenameChanged (const QString &)));
+    QObject::connect(editRecordingTemplateID3Title,  SIGNAL(textEdited(const QString &)), this, SLOT(slotRecordingTemplateID3TitleChanged (const QString &)));
+    QObject::connect(editRecordingTemplateID3Artist, SIGNAL(textEdited(const QString &)), this, SLOT(slotRecordingTemplateID3ArtistChanged(const QString &)));
+    QObject::connect(editRecordingTemplateID3Genre,  SIGNAL(textEdited(const QString &)), this, SLOT(slotRecordingTemplateID3GenreChanged (const QString &)));
 
 
     QObject::connect(checkboxAlarmDaily,    SIGNAL(toggled(bool)),               this, SLOT(slotSetDirty()));
@@ -99,7 +105,10 @@ TimeControlConfiguration::TimeControlConfiguration (QWidget *parent)
     QObject::connect(buttonDeleteAlarm,     SIGNAL(clicked()),                   this, SLOT(slotSetDirty()));
     QObject::connect(comboAlarmType,        SIGNAL(activated(int)),              this, SLOT(slotSetDirty()));
     QObject::connect(editSleep,             SIGNAL(valueChanged(int)),           this, SLOT(slotSetDirty()));
-    QObject::connect(editRecordingTemplate, SIGNAL(textEdited(const QString &)), this, SLOT(slotSetDirty()));
+    QObject::connect(editRecordingTemplateFileName,  SIGNAL(textEdited(const QString &)), this, SLOT(slotSetDirty()));
+    QObject::connect(editRecordingTemplateID3Title,  SIGNAL(textEdited(const QString &)), this, SLOT(slotSetDirty()));
+    QObject::connect(editRecordingTemplateID3Artist, SIGNAL(textEdited(const QString &)), this, SLOT(slotSetDirty()));
+    QObject::connect(editRecordingTemplateID3Genre,  SIGNAL(textEdited(const QString &)), this, SLOT(slotSetDirty()));
 }
 
 TimeControlConfiguration::~TimeControlConfiguration ()
@@ -139,7 +148,15 @@ bool TimeControlConfiguration::noticeAlarmsChanged(const AlarmVector &sl)
     idx = -1;
     int k = 0;
     for (ciAlarmVector i = alarms.begin(); i != alarms.end(); ++i, ++k) {
-        listAlarms->addItem(i->nextAlarm(true).toString());
+        const Alarm &alarm = *i;
+        QString      dateString = alarm.nextAlarm(true).toString();
+        dateString += alarm.isEnabled() ? "" : " (disabled)";
+        listAlarms->addItem(dateString);
+        QListWidgetItem *item = listAlarms->item(listAlarms->count()-1);
+        if (!m_defaultAlarmTextForegroundValid) {
+            m_enabledAlarmTextForeground = item->foreground();
+        }
+        item->setForeground(!alarm.isEnabled() ? m_disabledAlarmTextForeground : m_enabledAlarmTextForeground);
         if (i->ID() == currentID)
             idx = k;
     }
@@ -306,7 +323,13 @@ void TimeControlConfiguration::slotEnabledChanged( bool b)
 {
     int idx = listAlarms->currentRow();
     if (idx >= 0 && idx < alarms.size()) {
-        alarms[idx].setEnabled(b);
+        Alarm           &alarm      = alarms[idx];
+        QString          dateString = alarm.nextAlarm(true).toString();
+        QListWidgetItem *item       = listAlarms->item(idx);
+        alarm.setEnabled(b);
+        dateString += alarm.isEnabled() ? "" : " (disabled)";
+        item->setForeground(!alarm.isEnabled() ? m_disabledAlarmTextForeground : m_enabledAlarmTextForeground);
+        item->setText(dateString);
     }
 }
 
@@ -336,17 +359,59 @@ void TimeControlConfiguration::slotAlarmTypeChanged(int t)
     int idx = listAlarms->currentRow();
     if (idx >= 0 && idx < alarms.size()) {
         alarms[idx].setAlarmType((Alarm::AlarmType)t);
-        editRecordingTemplate ->setDisabled(t != Alarm::StartRecording);
-        labelRecordingTemplate->setDisabled(t != Alarm::StartRecording);
+        editRecordingTemplateFileName  ->setDisabled(t != Alarm::StartRecording);
+        editRecordingTemplateID3Title  ->setDisabled(t != Alarm::StartRecording);
+        editRecordingTemplateID3Artist ->setDisabled(t != Alarm::StartRecording);
+        editRecordingTemplateID3Genre  ->setDisabled(t != Alarm::StartRecording);
+        labelRecordingTemplate         ->setDisabled(t != Alarm::StartRecording);
+        labelRecordingTemplateFilename ->setDisabled(t != Alarm::StartRecording);
+        labelRecordingTemplateID3Title ->setDisabled(t != Alarm::StartRecording);
+        labelRecordingTemplateID3Artist->setDisabled(t != Alarm::StartRecording);
+        labelRecordingTemplateID3Genre ->setDisabled(t != Alarm::StartRecording);
     }
 }
 
 
-void TimeControlConfiguration::slotRecordingTemplateChanged(const QString &t)
+void TimeControlConfiguration::slotRecordingTemplateFilenameChanged(const QString &t)
 {
     int idx = listAlarms->currentRow();
     if (idx >= 0 && idx < alarms.size()) {
-        alarms[idx].setRecordingTemplate(t);
+        recordingTemplate_t tmp = alarms[idx].recordingTemplate();
+        tmp.filename = t;
+        alarms[idx].setRecordingTemplate(tmp);
+    }
+}
+
+
+void TimeControlConfiguration::slotRecordingTemplateID3TitleChanged(const QString &t)
+{
+    int idx = listAlarms->currentRow();
+    if (idx >= 0 && idx < alarms.size()) {
+        recordingTemplate_t tmp = alarms[idx].recordingTemplate();
+        tmp.id3Title = t;
+        alarms[idx].setRecordingTemplate(tmp);
+    }
+}
+
+
+void TimeControlConfiguration::slotRecordingTemplateID3ArtistChanged(const QString &t)
+{
+    int idx = listAlarms->currentRow();
+    if (idx >= 0 && idx < alarms.size()) {
+        recordingTemplate_t tmp = alarms[idx].recordingTemplate();
+        tmp.id3Artist = t;
+        alarms[idx].setRecordingTemplate(tmp);
+    }
+}
+
+
+void TimeControlConfiguration::slotRecordingTemplateID3GenreChanged(const QString &t)
+{
+    int idx = listAlarms->currentRow();
+    if (idx >= 0 && idx < alarms.size()) {
+        recordingTemplate_t tmp = alarms[idx].recordingTemplate();
+        tmp.id3Genre = t;
+        alarms[idx].setRecordingTemplate(tmp);
     }
 }
 
@@ -380,8 +445,15 @@ void TimeControlConfiguration::slotAlarmSelectChanged(int idx)
     labelStationSelection ->setDisabled(!valid);
     buttonDeleteAlarm     ->setDisabled(!valid);
     comboAlarmType        ->setDisabled(!valid);
-    editRecordingTemplate ->setDisabled(!valid || a.alarmType() != Alarm::StartRecording);
-    labelRecordingTemplate->setDisabled(!valid || a.alarmType() != Alarm::StartRecording);
+    editRecordingTemplateFileName  ->setDisabled(!valid || a.alarmType() != Alarm::StartRecording);
+    editRecordingTemplateID3Title  ->setDisabled(!valid || a.alarmType() != Alarm::StartRecording);
+    editRecordingTemplateID3Artist ->setDisabled(!valid || a.alarmType() != Alarm::StartRecording);
+    editRecordingTemplateID3Genre  ->setDisabled(!valid || a.alarmType() != Alarm::StartRecording);
+    labelRecordingTemplate         ->setDisabled(!valid || a.alarmType() != Alarm::StartRecording);
+    labelRecordingTemplateFilename ->setDisabled(!valid || a.alarmType() != Alarm::StartRecording);
+    labelRecordingTemplateID3Title ->setDisabled(!valid || a.alarmType() != Alarm::StartRecording);
+    labelRecordingTemplateID3Artist->setDisabled(!valid || a.alarmType() != Alarm::StartRecording);
+    labelRecordingTemplateID3Genre ->setDisabled(!valid || a.alarmType() != Alarm::StartRecording);
 
     editAlarmDate         ->setDate(a.alarmTime().date());
     editAlarmTime         ->setTime(a.alarmTime().time());
@@ -389,7 +461,10 @@ void TimeControlConfiguration::slotAlarmSelectChanged(int idx)
     checkboxAlarmEnable   ->setChecked(a.isEnabled());
     editAlarmVolume       ->setValue((int)rint(a.volumePreset() * 100));
     comboAlarmType        ->setCurrentIndex(a.alarmType());
-    editRecordingTemplate ->setText(a.recordingTemplate());
+    editRecordingTemplateFileName ->setText(a.recordingTemplate().filename);
+    editRecordingTemplateID3Title ->setText(a.recordingTemplate().id3Title);
+    editRecordingTemplateID3Artist->setText(a.recordingTemplate().id3Artist);
+    editRecordingTemplateID3Genre ->setText(a.recordingTemplate().id3Genre);
 
     int k = 0;
     const QString &sID = a.stationID();
