@@ -115,7 +115,7 @@ void   TimeShifter::restoreState (const KConfigGroup &config)
     QString mixerID = config.readEntry ("PlaybackMixerID", QString());
     QString channel = config.readEntry ("PlaybackMixerChannel", "PCM");
 
-    setPlaybackMixer(mixerID, channel);
+    setPlaybackMixer(mixerID, channel, /* force = */ true);
     setTempFile(fname, fsize);
 
     emit sigUpdateConfig();
@@ -133,6 +133,25 @@ ConfigPageInfo  TimeShifter::createConfigurationPage()
 }
 
 
+void TimeShifter::noticeConnectedSoundClient(ISoundStreamClient::thisInterface *i, bool pointer_valid)
+{
+    if (i && pointer_valid && i->getSoundStreamClientID() == m_PlaybackMixerID) {
+        setPlaybackMixer(m_PlaybackMixerID, m_PlaybackMixerChannel, /* force = */ true);
+    }
+}
+
+
+bool TimeShifter::noticePlaybackChannelsChanged(const QString & client_id, const QStringList &/*channels*/)
+{
+    if (client_id == m_PlaybackMixerID) {
+        setPlaybackMixer(m_PlaybackMixerID, m_PlaybackMixerChannel, /* force = */ true);
+    }
+    return true;
+}
+
+
+
+
 bool TimeShifter::noticeSoundStreamClosed(SoundStreamID id)
 {
     if (id == m_InputStreamID || ((id == m_OutputStreamSinkID) && (m_OutputStreamSinkID != m_OutputStreamSourceID))) {
@@ -485,18 +504,24 @@ bool TimeShifter::noticeReadyForPlaybackData(SoundStreamID id, size_t free_size)
         } else {
             while (!m_RingBuffer.error() && m_RingBuffer.getFillSize() > 0 && free_size > 0) {
                 if (m_PlaybackDataLeftInBuffer == 0) {
-                    char meta_buffer[1024];
-                    char prop_buffer[ 128];
-                    size_t &meta_size = *(size_t*)(void*)meta_buffer;
-                    size_t &prop_size = *(size_t*)(void*)prop_buffer;
+                    const size_t meta_size_max = 4*1024;
+                    const size_t prop_size_max = 1*1024;
+                    union {
+                        size_t meta_size;
+                        char   meta_buffer[meta_size_max];
+                    };
+                    union {
+                        size_t prop_size;
+                        char   prop_buffer[prop_size_max];
+                    };
                     meta_size = 0;
                     prop_size = 0;
 
-                    m_RingBuffer.takeData(meta_buffer, sizeof(meta_size));
+                    m_RingBuffer.takeData((char*)&meta_size, sizeof(meta_size));
                     if (meta_size && meta_size <= sizeof(meta_buffer)) {
                         m_RingBuffer.takeData(meta_buffer + sizeof(meta_size), meta_size - sizeof(meta_size));
                         readMetaDataFromBuffer(m_PlaybackMetaData, meta_buffer, meta_size);
-                    } else {
+                    } else if (meta_size > sizeof(meta_size)) {
                         m_RingBuffer.removeData(meta_size - sizeof(meta_size));
                     }
 
@@ -504,7 +529,7 @@ bool TimeShifter::noticeReadyForPlaybackData(SoundStreamID id, size_t free_size)
                     if (prop_size && prop_size <= sizeof(prop_buffer)) {
                         m_RingBuffer.takeData(prop_buffer + sizeof(prop_size), prop_size - sizeof(prop_size));
                         readCurrentStreamPropertiesFromBuffer(prop_buffer, prop_size);
-                    } else {
+                    } else if (prop_size > sizeof(prop_size)) {
                         m_RingBuffer.removeData(prop_size - sizeof(prop_size));
                     }
 
@@ -559,28 +584,49 @@ ISoundStreamClient *TimeShifter::searchPlaybackMixer()
 }
 
 
-bool  TimeShifter::setPlaybackMixer(const QString &soundStreamClientID, const QString &ch)
+static inline void assignChannelIfValid(QString &dest_channel, const QString &test_channel, const QStringList &valid_channels)
 {
-    m_PlaybackMixerID = soundStreamClientID;
-    m_PlaybackMixerChannel = ch;
-
-    ISoundStreamClient *playback_mixer = searchPlaybackMixer();
-
-    float  oldVolume;
-    if (m_OutputStreamSinkID.isValid()) {
-        queryPlaybackVolume(m_OutputStreamSinkID, oldVolume);
-        sendStopPlayback(m_OutputStreamSinkID);
-        sendReleasePlayback(m_OutputStreamSinkID);
+    if (valid_channels.contains(test_channel) || !valid_channels.size()) {
+        dest_channel = test_channel;
     }
+}
 
-    if (playback_mixer)
-        playback_mixer->preparePlayback(m_OutputStreamSinkID, m_PlaybackMixerChannel, /*active*/true, /*start_imm*/false);
+bool  TimeShifter::setPlaybackMixer(QString soundStreamClientID, QString ch, bool force)
+{
+    QString old_channel           = m_PlaybackMixerChannel;
+    m_PlaybackMixerID             = soundStreamClientID;
+    ISoundStreamClient *mixer     = searchPlaybackMixer();
+    QStringList         channels  = mixer ? mixer->getPlaybackChannels() : QStringList();
 
-    if (m_OutputStreamSinkID.isValid()) {
-        sendStartPlayback(m_OutputStreamSinkID);
-        sendPlaybackVolume(m_OutputStreamSinkID, oldVolume);
+    if (channels.size()) {
+        assignChannelIfValid(m_PlaybackMixerChannel, channels[0], channels);  // lowest priority
     }
+    assignChannelIfValid(m_PlaybackMixerChannel, "PCM",       channels);
+    assignChannelIfValid(m_PlaybackMixerChannel, "Wave",      channels);
+    assignChannelIfValid(m_PlaybackMixerChannel, "Master",    channels);
+    assignChannelIfValid(m_PlaybackMixerChannel, ch,          channels);  // highest priority
 
+    bool change = (m_PlaybackMixerID != soundStreamClientID) || (old_channel != m_PlaybackMixerChannel);
+
+    if (change || force) {
+        float  oldVolume = -1;
+        if (m_OutputStreamSinkID.isValid()) {
+            queryPlaybackVolume(m_OutputStreamSinkID, oldVolume);
+            sendStopPlayback(m_OutputStreamSinkID);
+            sendReleasePlayback(m_OutputStreamSinkID);
+        }
+
+        if (mixer)
+            mixer->preparePlayback(m_OutputStreamSinkID, m_PlaybackMixerChannel, /*active*/true, /*start_imm*/false);
+
+        if (m_OutputStreamSinkID.isValid()) {
+            sendStartPlayback(m_OutputStreamSinkID);
+            sendPlaybackVolume(m_OutputStreamSinkID, oldVolume);
+        }
+
+        if (change)
+            emit sigUpdateConfig();
+    }
     return true;
 }
 
